package scacchi.main;

import scacchi.main.ArgomentiComando.*;

/**
 * Classe contenitore per le eccezioni di partita. <br>
 * La classe contiene solo static nested classes e non pu&ograve;
 * essere istanziata o estesa.
 * @author Stefano Guerrini
 * @version 04/12/10
 */
public final class EccezioniPartita {

    /** Questa classe non pu&ograve; essere istanziata. */
    private EccezioniPartita() {};

    /** Radice della gerachia di eccezioni della partita. */
    public static class PartitaException extends Exception {
	public PartitaException(String s) { super(s); }
    }

    /** 
     * Eccezione causata dal fatto che si &egrave; cercato di cominciare
     * una partita da una posizione non valida.
     */
    public static class PosizioneInizialeException extends PartitaException {
	/**
	 * @param l   La lista delle posizioni di pezzi della posizione
	 *            errata.
	 * @param err Il problema che rende la posizione non valida.
	 */
	public PosizioneInizialeException(ListaPosPezziTesto l, String err) {
	    super("La posizione\n'" + l + "'non e' corretta:\n" + err);
	}
    }

    /** 
     * Eccezione causata dal fatto che una mossa non pu&ograve; 
     * essere eseguita.
     */
    public static class MossaException extends PartitaException {
	/**
	 * @param s   La mossa che si cercava di eseguire.
	 * @param err Il problema che impedisce la mossa.
	 */
	public MossaException(MossaTesto s, String err) {
	    super("La mossa '" + s + "' non puo' essere eseguita: " + err);
	}
    }

    /**
     * Eccezione causata dal fatto che una mossa non pu&ograve;
     * essere eseguita perch&egrave; impossibile.
     * @see scacchi.main.EccezioniPartita.MossaException
     */
    public static class MossaImpossibileException extends MossaException {
	/** @param s La mossa che si cercava di eseguire. */
	public MossaImpossibileException(MossaTesto s) {
	    super(s, "Mossa impossibile");
	}
    }

    /**
     * Eccezione causata dal fatto che una mossa non pu&ograve; 
     * essere eseguita perch&egrave; non valida. <br>
     * La mossa sarebbe possibile in base alle regole di movimento dei pezzi,
     * ma non &egrave; valida a causa di qualche vincolo non rispettato
     * (ad esempio, il proprio Re viene lasciato o messo sotto scacco.
     * @see scacchi.main.EccezioniPartita.MossaException
     */
    public static class MossaNonValidaException extends MossaException {
	/**
	 * @param s   La mossa che si cercava di eseguire.
	 * @param err Il problema specificio per cui la mossa non
	 *            &egrave; valida.
	 */
	public MossaNonValidaException(MossaTesto s, String err) {
	    super(s, "Mossa non valida (" + err + ")");
	}
    }

    /**
     * Eccezione causata dal fatto che si &egrave; assunto che la casa
     * conteneva un pezzo.
     */
    public static class CasaVuotaException extends PartitaException {
	/** @param s La casa vuota. */
	public CasaVuotaException(CasaTesto s) {
	    super("La casa " + "'" + s + "'" + "e' vuota");
	}
    }

    /*
     * Eccezione causata dal fatto che si &egrave; cercato di tornare
     * indietro o di leggere mosse oltre la posizione iniziale.
     */
    public static class NoMoreMosseException extends PartitaException {
	public NoMoreMosseException() {
	    super("Si vuole tornare indietro oltre la posizione iniziale");
	}
    }

    /**
     * Eccezione causata dal fatto che una casa contiene un pezzo di colore
     * diverso da quello che si stava assumendo.
     */
    public static class ColorePezzoException extends PartitaException {
	/**
	 * @param s      La casa in esame.
	 * @param colore Il colore del pezzo che ci si aspettava di trovare.
	 */
	public ColorePezzoException(CasaTesto s, boolean colore) {
	    super("Il pezzo nella casa " + "'" + s + "'" + "e' di colore "
		  + ((colore == Partita.BIANCO) ? "nero" : "bianco"));
	}
    }
}
