package scacchi.main;

import scacchi.main.EccezioniPartita.*;
import scacchi.main.ArgomentiComando.*;
import scacchi.main.EccezioniTesto.*;

/**
 * @author Stefano Guerrini
 * @version 04/12/07
 * Interfaccia da implementare per realizzare il progetto. <br>
 */
public interface Partita {
    /*
     * Valore booleano associato al colore BIANCO.
     * @see #NERO
     */
    public static final boolean BIANCO = true;
    /*
     * Valore booleano associato al colore NERO.
     * @see #BIANCO
     */
    public static final boolean NERO = false;

    /**
     * Restituisce il numero di mossa corrente. <br>
     * Le mosse vanno contate tenendo conto che una mossa completa si
     * compone di una (semi-)mossa del bianco e di una (semi-)mossa
     * del nero.
     * @return Il numero della mossa. Un valore positivo del
     *         indica che la mossa sta al BIANCO, un valore negativo
     *         che la mossa sta al NERO
     */
    public int getNumMossa();

    /**
     * Esegue una mossa
     * @param m la mossa da giocare
     * @throws scacchi.main.EccezioniPartita.MossaImpossibileException
     *         Se la mossa non corrisponde a un movimento possibile
     * @throws scacchi.main.EccezioniPartita.MossaNonValidaException
     *         Se la mossa, anche se possibile, non &egrave; giocabile
     *         perch&egrave; il re rimane o finisce sotto scacco,
     *         oppure si tratta di un arrocco o di una presa en-passant
     *         non valida.
     */
    public ListaPosPezziTesto eseguiMossa(MossaTesto m)
	throws MossaException;

    /**
     * Torna indietro di una (semi-)mossa
     * @return la nuova posizione dopo che si &egrave; tornati indietro
     * @throws scacchi.main.EccezioniPartita.NoMoreMosseException
     *         Se la partita &egrave; ancors nella posizione iniziale
     */
    public ListaPosPezziTesto tornaIndietro()
	throws NoMoreMosseException;

    /**
     * Torna indietro di un certo numero di semi-mosse o tratti
     * @return la nuova posizione dopo che si &egrave; tornati indietro
     * @throws scacchi.main.EccezioniPartita.NoMoreMosseException
     *         Se si cerca di tornare indietro oltre la posizione iniziale
     */
    public ListaPosPezziTesto tornaIndietro(int n)
	throws NoMoreMosseException;

    /**
     * Elenca le mosse della partita
     * @return la sequenza delle mosse
     */
    public ListaMosseTesto listaMosse();

    /**
     * Elenca le ultime semi-mosse della partita
     * @param n Le semi-mosse da elencare (una semi-mossa &egrave; la parte
     *          di mossa di uno dei due giocatori)
     * @return La sequenza delle ultime semi-mosse giocate
     * @throws scacchi.main.EccezioniPartita.NoMoreMosseException
     *         Se sono state giocate meno di n semi-mosse
     */
    public ListaMosseTesto listaMosse(int n)
	throws NoMoreMosseException;

    /**
     * Restituisce la posizione dei pezzi
     */
    public ListaPosPezziTesto getPosizionePezzi();

    /**
     * Restituisce l'elenco delle case controllate da un pezzo. <br>
     * Una casa &egrave; controllata da un pezzo se il pezzo pu&ograve;
     * arrivarci in base alle sue regole di movimento e se la cosa &egrave;
     * vuota o &egrave; occupata da un pezzo avversario. <br>
     * Le case controllate da un pezzo non corrispondono necessariamente
     * alle mosse possibili del pezzo. Un pedone, pu&ograve; muovere
     * nella casa immediatamente di fronte alla sua posizione corrente ma
     * non la controlla, visto che non pu&ograve; catturarci alcun pezzo.
     * Analogamente, un pedone cotrolla le case in cui pu&ograve; effettuare
     * una cattura, ma non pu&ograve; in tali case se sono vuote.
     * non possono essere eseguite perch&egrave; il re &egrave; sotto scacco
     * @param c casa del pezzo da muovere
     * @throws scacchi.main.EccezioniPartita.CasaVuotaException
     *         Nel caso in cui la casa s &egrave; vuota
     */
    public ListaCaseTesto caseControllate(CasaTesto c)
	throws CasaVuotaException;

    /**
     * Restituisce l'elenco delle case controllate dai pezzi di un
     * giocatore. <br>
     * @param colore Il colore del giocatore da analizzare
     * @see #caseControllate(ArgomentiComando.CasaTesto)
     */
    public ListaCaseTesto caseControllate(boolean colore);

    /**
     * Restituisce l'elenco delle case controllate dal giocatore
     * che ha il tratto. <br>
     * @see #caseControllate(ArgomentiComando.CasaTesto)
     */
    public ListaCaseTesto caseControllate();

    /**
     * Elenca i pezzi di un colore che controllano una casa
     * @param c      La casa che si vuole analizzare
     * @param colore Il colore dei pezzi da considerare
     * @return La lista delle posizioni dei pezzi che controllano la casa.
     * @see #caseControllate(ArgomentiComando.CasaTesto)
     * @see #caseControllate(boolean)
     */
    public ListaPosPezziTesto casaControllataDa(CasaTesto c, boolean colore);

    /**
     * Restituisce l'elenco delle mosse possibili di un pezzo del giocatore
     * che ha la mossa. <br>
     * Questo metodo non tiene conto di un eventuale scacco al re
     * elenca cio&egrave; tutti i possibili movimenti di pezzi, anche
     * quelli che non eliminano la situazione di scacco. <br>
     * Il metodo {@link #getMosseValide(ArgomentiComando.CasaTesto)}
     * ritorna invece l'elenco delle sole mosse valide e quindi realmente
     * eseguibili.
     * @param c La casa del pezzo da muovere
     * @throws scacchi.main.EccezioniPartita.CasaVuotaException
     *         Nel caso in cui la casa s &egrave; vuota.
     * @throws scacchi.main.EccezioniPartita.ColorePezzoException
     *         Nel caso in cui la casa s contiene un pezzo del colore
     *         del giocatore che non ha la mossa.
     * @see #getMossePossibili()
     * @see #getMosseValide()
     * @see #getMosseValide(ArgomentiComando.CasaTesto)
     */
    public ListaCaseTesto getMossePossibili(CasaTesto c)
	throws PartitaException;

    /**
     * Restituisce l'elenco delle mosse possibili per il giocatore a
     * cui sta la mossa. <br>
     * @see #getMossePossibili(ArgomentiComando.CasaTesto)
     * @see #getMosseValide()
     * @see #getMosseValide(ArgomentiComando.CasaTesto)
     */
    public ListaMosseTesto getMossePossibili();

    /**
     * Restituisce l'elenco delle mosse valide di un pezzo del
     * giocatore che deve muovere. <br>
     * Ritorna solo le mosse che non lasciano o mettono il
     * re sotto scacco. <br>
     * Il metodo {@link #getMossePossibili(ArgomentiComando.CasaTesto)}
     * ritorna invece l'elenco delle mosse possibili.
     * @param c La casa del pezzo da muovere
     * @throws scacchi.main.EccezioniPartita.CasaVuotaException
     *         Nel caso in cui la casa s &egrave; vuota.
     * @throws scacchi.main.EccezioniPartita.ColorePezzoException
     *         Nel caso in cui la casa s contiene un pezzo del colore
     *         del giocatore che non ha la mossa.
     * @see #getMosseValide()
     * @see #getMossePossibili()
     * @see #getMossePossibili(ArgomentiComando.CasaTesto)
     */
    public ListaMosseTesto getMosseValide(CasaTesto c);

    /**
     * Restituisce l'elenco delle mosse valide per il giocatore che deve
     * muovere. <br>
     * Ritorna solo le mosse che non lasciano o mettono il
     * re sotto scacco. <br>
     * Se non ci sono mosse possibili allora si ha una situazione di
     * scacco matto, se il re era sotto scacco, o di stallo, se non lo era.
     * @see #getMosseValide(ArgomentiComando.CasaTesto)
     * @see #getMossePossibili()
     * @see #getMossePossibili(ArgomentiComando.CasaTesto)
     */
    public ListaMosseTesto getMosseValide();


    /**
     * Verifica se il re del giocatore che ha il tratto &egrave; sotto scacco.
     * @return true se il re &egrave; sotto scacco, false altrimenti.
     */
    public boolean scaccoAlRe();
}
